@@ -6,6 +6,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <sys/types.h>
 
 #ifdef HAVE_DIRENT_H
@@ -243,8 +244,25 @@ void findcrashfiles()
       sprintf(_STR, "%s/%s", xobjs.tempdir, dp->d_name);
       snptr = _STR + strlen(xobjs.tempdir) + 1;
       if (!strncmp(snptr, "XC", 2)) {
+	 char *dotptr = strchr(snptr, '.');
+	 pid = -1;
+	 if (dotptr && dotptr > snptr + 2) {
+	    *dotptr = '\0';
+	    if (sscanf(snptr + 2, "%d", &pid) != 1) 
+	       pid = -1;
+	    *dotptr = '.';
+         }
          if ((!stat(_STR, &sbuf)) && (sbuf.st_uid == userid)) {
 	    if ((recent == 0) || (sbuf.st_ctime > recent)) {
+
+	       /* Check if the PID belongs to an active process	*/
+	       /* by sending a CONT signal and checking if 	*/
+	       /* there was no error result.			*/
+
+	       if (pid != -1)
+		  if (kill((pid_t)pid, SIGCONT) == 0)
+		     continue;
+
 	       recent = sbuf.st_ctime;
 	       strcpy(_STR2, _STR);
 	    }
