@@ -38,8 +38,7 @@
 
 extern Display *dpy;
 extern Pixmap STIPPLE[8];
-extern short eventmode;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern int *appcolors;
 extern short textpos, textend;
@@ -348,28 +347,28 @@ short checkbounds()
 
    /* check window-to-user space */
 
-   lval = 2 * (long)((float) (areastruct.width) / (*areastruct.vscale)) +
-	(long)areastruct.pcorner->x;
+   lval = 2 * (long)((float) (areawin->width) / (*areawin->vscale)) +
+	(long)areawin->pcorner->x;
    if (lval != (long)((short)lval)) return -1;
-   lval = 2 * (long)((float) (areastruct.height) / (*areastruct.vscale)) +
-	(long)areastruct.pcorner->y;
+   lval = 2 * (long)((float) (areawin->height) / (*areawin->vscale)) +
+	(long)areawin->pcorner->y;
    if (lval != (long)((short)lval)) return -1;
 
    /* check user-to-window space */
 
-   lval = (long)((float)(topobject->bbox.lowerleft.x - areastruct.pcorner->x) *
-	(*areastruct.vscale));
+   lval = (long)((float)(topobject->bbox.lowerleft.x - areawin->pcorner->x) *
+	(*areawin->vscale));
    if (lval != (long)((short)lval)) return -1;
-   lval = (long)areastruct.height - (long)((float)(topobject->bbox.lowerleft.y -
-	areastruct.pcorner->y) * (*areastruct.vscale)); 
+   lval = (long)areawin->height - (long)((float)(topobject->bbox.lowerleft.y -
+	areawin->pcorner->y) * (*areawin->vscale)); 
    if (lval != (long)((short)lval)) return -1;
    UTransformbyCTM(DCTM, &(topobject->bbox.lowerleft), &testpt, 1);
 
    lval = (long)((float)(topobject->bbox.lowerleft.x + topobject->bbox.width -
-	areastruct.pcorner->x) * (*areastruct.vscale));
+	areawin->pcorner->x) * (*areawin->vscale));
    if (lval != (long)((short)lval)) return -1;
-   lval = (long)areastruct.height - (long)((float)(topobject->bbox.lowerleft.y +
-	topobject->bbox.height - areastruct.pcorner->y) * (*areastruct.vscale)); 
+   lval = (long)areawin->height - (long)((float)(topobject->bbox.lowerleft.y +
+	topobject->bbox.height - areawin->pcorner->y) * (*areawin->vscale)); 
    if (lval != (long)((short)lval)) return -1;
 
    return 0;
@@ -383,9 +382,9 @@ void window_to_user(short xw, short yw, XPoint *upt)
 {
   float tmpx, tmpy;
 
-  tmpx = (float)xw / (*areastruct.vscale) + (float)areastruct.pcorner->x;
-  tmpy = (float)(areastruct.height - yw) / (*areastruct.vscale) + 
-	(float)areastruct.pcorner->y;
+  tmpx = (float)xw / (*areawin->vscale) + (float)areawin->pcorner->x;
+  tmpy = (float)(areawin->height - yw) / (*areawin->vscale) + 
+	(float)areawin->pcorner->y;
 
   tmpx += (tmpx > 0) ? 0.5 : -0.5;
   tmpy += (tmpy > 0) ? 0.5 : -0.5;
@@ -402,9 +401,9 @@ void user_to_window(XPoint upt, XPoint *wpt)
 {
   float tmpx, tmpy;
 
-  tmpx = (float)(upt.x - areastruct.pcorner->x) * (*areastruct.vscale);
-  tmpy = (float)areastruct.height - (float)(upt.y - areastruct.pcorner->y)
-	* (*areastruct.vscale); 
+  tmpx = (float)(upt.x - areawin->pcorner->x) * (*areawin->vscale);
+  tmpy = (float)areawin->height - (float)(upt.y - areawin->pcorner->y)
+	* (*areawin->vscale); 
 
   tmpx += (tmpx > 0) ? 0.5 : -0.5;
   tmpy += (tmpy > 0) ? 0.5 : -0.5;
@@ -442,13 +441,13 @@ XPoint UGetCursor()
    XPoint newpos;
  
 #ifdef TCL_WRAPPER
-   /* Don't use areastruct.areawin;  if called from inside an object	*/
-   /* (e.g., "here" in a Tcl expression), areastruct.areawin will be	*/
+   /* Don't use areawin->window;  if called from inside an object	*/
+   /* (e.g., "here" in a Tcl expression), areawin->window will be	*/
    /* an off-screen pixmap, and cause a crash.				*/
-   XQueryPointer(dpy, Tk_WindowId(areastruct.area), &nullwin, &nullwin,
+   XQueryPointer(dpy, Tk_WindowId(areawin->area), &nullwin, &nullwin,
 	&nullint, &nullint, &xpos, &ypos, &nullui);
 #else
-   XQueryPointer(dpy, areastruct.areawin, &nullwin, &nullwin, &nullint,
+   XQueryPointer(dpy, areawin->window, &nullwin, &nullwin, &nullint,
 	&nullint, &xpos, &ypos, &nullui);
 #endif
 
@@ -483,22 +482,22 @@ void u2u_snap(XPoint *uvalue)
    float tmpx, tmpy;
    float tmpix, tmpiy;
 
-   if (areastruct.snapto) {
-      tmpx = (float)uvalue->x / xobjs.pagelist[areastruct.page]->snapspace;
+   if (areawin->snapto) {
+      tmpx = (float)uvalue->x / xobjs.pagelist[areawin->page]->snapspace;
       if (tmpx > 0)
 	 tmpix = (float)((int)(tmpx + 0.5));
       else
          tmpix = (float)((int)(tmpx - 0.5));
 
-      tmpy = (float)uvalue->y / xobjs.pagelist[areastruct.page]->snapspace;
+      tmpy = (float)uvalue->y / xobjs.pagelist[areawin->page]->snapspace;
       if (tmpy > 0)
          tmpiy = (float)((int)(tmpy + 0.5));
       else
          tmpiy = (float)((int)(tmpy - 0.5));
 
-      tmpix *= xobjs.pagelist[areastruct.page]->snapspace;
+      tmpix *= xobjs.pagelist[areawin->page]->snapspace;
       tmpix += (tmpix > 0) ? 0.5 : -0.5;
-      tmpiy *= xobjs.pagelist[areastruct.page]->snapspace;
+      tmpiy *= xobjs.pagelist[areawin->page]->snapspace;
       tmpiy += (tmpiy > 0) ? 0.5 : -0.5;
 
       uvalue->x = (int)tmpix;
@@ -572,14 +571,14 @@ void UCopyCTM(fctm, tctm)
 
 void UMakeWCTM(Matrix *ctm)
 {
-   ctm->a *= (*areastruct.vscale);
-   ctm->b *= (*areastruct.vscale);
-   ctm->c = (ctm->c - (float)areastruct.pcorner->x) * (*areastruct.vscale);
+   ctm->a *= (*areawin->vscale);
+   ctm->b *= (*areawin->vscale);
+   ctm->c = (ctm->c - (float)areawin->pcorner->x) * (*areawin->vscale);
    
-   ctm->d *= -(*areastruct.vscale);
-   ctm->e *= -(*areastruct.vscale);
-   ctm->f = (float)areastruct.height + ((float)areastruct.pcorner->y - ctm->f) *
-	    (*areastruct.vscale);
+   ctm->d *= -(*areawin->vscale);
+   ctm->e *= -(*areawin->vscale);
+   ctm->f = (float)areawin->height + ((float)areawin->pcorner->y - ctm->f) *
+	    (*areawin->vscale);
 }
 
 /*------------------------------------------------------------------------*/
@@ -760,13 +759,13 @@ void UPopCTM()
 {
    Matrixptr lastmatrix;
 
-   if (areastruct.MatStack == NULL) {
+   if (areawin->MatStack == NULL) {
       Wprintf("Matrix stack pop error");
       return;
    }
-   lastmatrix = areastruct.MatStack->nextmatrix;
-   free(areastruct.MatStack);
-   areastruct.MatStack = lastmatrix;
+   lastmatrix = areawin->MatStack->nextmatrix;
+   free(areawin->MatStack);
+   areawin->MatStack = lastmatrix;
 }
 
 /*------------------------------------------------------------------------*/
@@ -776,12 +775,12 @@ void UPushCTM()
    Matrixptr nmatrix;
 
    nmatrix = (Matrixptr)malloc(sizeof(Matrix));
-   if (areastruct.MatStack == NULL)
+   if (areawin->MatStack == NULL)
       UResetCTM(nmatrix);
    else
-      UCopyCTM(areastruct.MatStack, nmatrix); 
-   nmatrix->nextmatrix = areastruct.MatStack;
-   areastruct.MatStack = nmatrix;
+      UCopyCTM(areawin->MatStack, nmatrix); 
+   nmatrix->nextmatrix = areawin->MatStack;
+   areawin->MatStack = nmatrix;
 }
 
 /*------------------------------------------------------------------------*/
@@ -958,7 +957,7 @@ void calcinstbbox(genericptr *bboxgen, short *llx, short *lly, short *urx,
    *llx = *lly = 32767;
    *urx = *ury = -32768;
 
-   calcbboxsingle(bboxgen, areastruct.topinstance, llx, lly, urx, ury);
+   calcbboxsingle(bboxgen, areawin->topinstance, llx, lly, urx, ury);
 }
 
 /*----------------------------------------------------------------------*/
@@ -1217,7 +1216,7 @@ void updateinstparam(objectptr bobj)
 void calcbbox(objinstptr binst)
 {
    calcbboxvalues(binst, (genericptr *)NULL);
-   if (binst == areastruct.topinstance) {
+   if (binst == areawin->topinstance) {
       updatepagebounds(topobject);
    }
 }
@@ -1230,7 +1229,7 @@ void calcbbox(objinstptr binst)
 
 void singlebbox(genericptr *gelem)
 {
-   calcbboxvalues(areastruct.topinstance, (genericptr *)gelem);
+   calcbboxvalues(areawin->topinstance, (genericptr *)gelem);
    updatepagebounds(topobject);
 }
 
@@ -1241,9 +1240,9 @@ void singlebbox(genericptr *gelem)
 void calcbboxselect()
 {
    short *bsel;
-   for (bsel = areastruct.selectlist; bsel < areastruct.selectlist +
-		areastruct.selects; bsel++)
-      calcbboxvalues(areastruct.topinstance, topobject->plist + *bsel);
+   for (bsel = areawin->selectlist; bsel < areawin->selectlist +
+		areawin->selects; bsel++)
+      calcbboxvalues(areawin->topinstance, topobject->plist + *bsel);
   
    updatepagebounds(topobject);
 }
@@ -1350,15 +1349,15 @@ void centerview(objinstptr tinst)
    width = corner.x - origin.x;
    height = corner.y - origin.y;
 
-   fitwidth = (float)areastruct.width / ((float)width + 2 * DEFAULTGRIDSPACE);
-   fitheight = (float)areastruct.height / ((float)height + 2 * DEFAULTGRIDSPACE);
+   fitwidth = (float)areawin->width / ((float)width + 2 * DEFAULTGRIDSPACE);
+   fitheight = (float)areawin->height / ((float)height + 2 * DEFAULTGRIDSPACE);
 
    tobj->viewscale = (fitwidth < fitheight) ?
 		 min(MINAUTOSCALE, fitwidth) : min(MINAUTOSCALE, fitheight);
 
-   tobj->pcorner.x = origin.x - (areastruct.width
+   tobj->pcorner.x = origin.x - (areawin->width
 	       / tobj->viewscale - width) / 2;
-   tobj->pcorner.y = origin.y - (areastruct.height
+   tobj->pcorner.y = origin.y - (areawin->height
 	       / tobj->viewscale - height) / 2;
 }
 
@@ -1369,10 +1368,10 @@ void centerview(objinstptr tinst)
 void refresh(xcWidget bw, caddr_t clientdata, caddr_t calldata)
 {
    drawarea(NULL, NULL, NULL);
-   if (areastruct.scrollbarh)
-      drawhbar(areastruct.scrollbarh, NULL, NULL);
-   if (areastruct.scrollbarv)
-      drawvbar(areastruct.scrollbarv, NULL, NULL);
+   if (areawin->scrollbarh)
+      drawhbar(areawin->scrollbarh, NULL, NULL);
+   if (areawin->scrollbarv)
+      drawvbar(areawin->scrollbarv, NULL, NULL);
    printname(topobject);
 }
 
@@ -1386,9 +1385,9 @@ void zoomview(xcWidget w, caddr_t clientdata, caddr_t calldata)
 	 eventmode == MOVE_MODE || eventmode == CATALOG_MODE ||
 	eventmode == FONTCAT_MODE || eventmode == EFONTCAT_MODE) {
 
-      centerview(areastruct.topinstance);
+      centerview(areawin->topinstance);
       invalidate_graphics(topobject);
-      areastruct.lastbackground = NULL;
+      areawin->lastbackground = NULL;
       renderbackground();
       refresh(NULL, NULL, NULL);
    }
@@ -1405,7 +1404,7 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
    UTransformbyCTM(DCTM, pt1, &newpt1, 1);
    UTransformbyCTM(DCTM, pt2, &newpt2, 1);
 
-   DrawLine(dpy, areastruct.areawin, areastruct.gc,
+   DrawLine(dpy, areawin->window, areawin->gc,
 		newpt1.x, newpt1.y, newpt2.x, newpt2.y); 
 } 
 
@@ -1413,9 +1412,9 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
 
 void UDrawLine(XPoint *pt1, XPoint *pt2)
 {
-   float tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
+   float tmpwidth = UTopTransScale(xobjs.pagelist[areawin->page]->wirewidth);
 
-   SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid, CapRound, JoinBevel);
+   SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid, CapRound, JoinBevel);
    UDrawSimpleLine(pt1, pt2); 
 } 
 
@@ -1438,23 +1437,23 @@ void UDrawCircle(XPoint *upt, u_char which)
    XPoint wpt;
 
    user_to_window(*upt, &wpt);
-   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapButt, JoinMiter);
 
    switch(which) {
       case P_POSITION_X:
-         XDrawArc(dpy, areastruct.areawin, areastruct.gc, wpt.x - 4,
+         XDrawArc(dpy, areawin->window, areawin->gc, wpt.x - 4,
 		wpt.y - 4, 8, 8, -(45 * 64), (90 * 64));
-         XDrawArc(dpy, areastruct.areawin, areastruct.gc, wpt.x - 4,
+         XDrawArc(dpy, areawin->window, areawin->gc, wpt.x - 4,
 		wpt.y - 4, 8, 8, (135 * 64), (90 * 64));
 	 break;
       case P_POSITION_Y:
-         XDrawArc(dpy, areastruct.areawin, areastruct.gc, wpt.x - 4,
+         XDrawArc(dpy, areawin->window, areawin->gc, wpt.x - 4,
 		wpt.y - 4, 8, 8, (45 * 64), (90 * 64));
-         XDrawArc(dpy, areastruct.areawin, areastruct.gc, wpt.x - 4,
+         XDrawArc(dpy, areawin->window, areawin->gc, wpt.x - 4,
 		wpt.y - 4, 8, 8, (225 * 64), (90 * 64));
 	 break;
       default:
-         XDrawArc(dpy, areastruct.areawin, areastruct.gc, wpt.x - 4,
+         XDrawArc(dpy, areawin->window, areawin->gc, wpt.x - 4,
 		wpt.y - 4, 8, 8, 0, (360 * 64));
 	 break;
    }
@@ -1466,10 +1465,10 @@ void UDrawCircle(XPoint *upt, u_char which)
 
 void UDrawXAt(XPoint *wpt)
 {
-   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x - 3,
+   SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapButt, JoinMiter);
+   DrawLine(dpy, areawin->window, areawin->gc, wpt->x - 3,
 		wpt->y - 3, wpt->x + 3, wpt->y + 3);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x + 3,
+   DrawLine(dpy, areawin->window, areawin->gc, wpt->x + 3,
 		wpt->y - 3, wpt->x - 3, wpt->y + 3);
 }
 
@@ -1606,13 +1605,13 @@ void UDrawTextLine(labelptr curlabel, short tpos)
    UPreMultCTM(DCTM, curlabel->position, curlabel->scale, curlabel->rotation);
    tmpjust = flipadjust(curlabel->justify);
 
-   SetFunction(dpy, areastruct.gc, GXxor);
-   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areawin->gc, GXxor);
+   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
 
-   tmpext = ULength(curlabel->string, areastruct.topinstance, 0.0, tpos, NULL);
+   tmpext = ULength(curlabel->string, areawin->topinstance, 0.0, tpos, NULL);
    xdist = tmpext.width;
    xbase = tmpext.base;
-   tmpext = ULength(curlabel->string, areastruct.topinstance, 0.0, 0, NULL);
+   tmpext = ULength(curlabel->string, areawin->topinstance, 0.0, 0, NULL);
 
    points[0].x = (tmpjust & NOTLEFT ?
         (tmpjust & RIGHT ? -tmpext.width : -tmpext.width >> 1) : 0)
@@ -1656,23 +1655,23 @@ void UDrawXLine(XPoint opt, XPoint cpt)
 {
    XPoint upt, vpt;
 
-   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
-   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areawin->gc, GXxor);
 
    user_to_window(cpt, &upt);
    user_to_window(opt, &vpt);
 
-   SetThinLineAttributes(dpy, areastruct.gc, 0, LineOnOffDash, CapButt, JoinMiter);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, vpt.x, vpt.y, upt.x, upt.y);
+   SetThinLineAttributes(dpy, areawin->gc, 0, LineOnOffDash, CapButt, JoinMiter);
+   DrawLine(dpy, areawin->window, areawin->gc, vpt.x, vpt.y, upt.x, upt.y);
 
-   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x - 3, upt.y - 3,
+   SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapButt, JoinMiter);
+   DrawLine(dpy, areawin->window, areawin->gc, upt.x - 3, upt.y - 3,
 		upt.x + 3, upt.y + 3);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x + 3, upt.y - 3,
+   DrawLine(dpy, areawin->window, areawin->gc, upt.x + 3, upt.y - 3,
 		upt.x - 3, upt.y + 3);
 
-   SetFunction(dpy, areastruct.gc, areastruct.gctype);
-   SetForeground(dpy, areastruct.gc, areastruct.gccolor);
+   SetFunction(dpy, areawin->gc, areawin->gctype);
+   SetForeground(dpy, areawin->gc, areawin->gccolor);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1684,17 +1683,17 @@ void UDrawBox(XPoint origin, XPoint corner)
    user_to_window(origin, &worig);
    user_to_window(corner, &wcorn);
 
-   SetFunction(dpy, areastruct.gc, GXxor);
-   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
-   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
+   SetFunction(dpy, areawin->gc, GXxor);
+   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+   SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
 
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
+   DrawLine(dpy, areawin->window, areawin->gc, worig.x, worig.y,
 		worig.x, wcorn.y);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
+   DrawLine(dpy, areawin->window, areawin->gc, worig.x, wcorn.y,
 		wcorn.x, wcorn.y);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
+   DrawLine(dpy, areawin->window, areawin->gc, wcorn.x, wcorn.y,
 		wcorn.x, worig.y);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
+   DrawLine(dpy, areawin->window, areawin->gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
 
@@ -1715,12 +1714,12 @@ void UDrawRescaleBox(XPoint *corner)
    /* To be done---draw a box around each selected part, not	*/
    /* just the first one.					*/
 
-   SetFunction(dpy, areastruct.gc, GXxor);
-   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
-   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
+   SetFunction(dpy, areawin->gc, GXxor);
+   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+   SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
    
-   for (j = 0; j < areastruct.selects; j++) {
-      rgen = SELTOGENERIC(areastruct.selectlist + j);
+   for (j = 0; j < areawin->selects; j++) {
+      rgen = SELTOGENERIC(areawin->selectlist + j);
 
       switch(ELEMENTTYPE(rgen)) {
          case LABEL:
@@ -1730,7 +1729,7 @@ void UDrawRescaleBox(XPoint *corner)
 	    mindist = LONG_MAX;
 	    while (mindist > 4) {
 	       rlab->scale = newscale;
-	       labelbbox(rlab, newpoints, areastruct.topinstance);
+	       labelbbox(rlab, newpoints, areawin->topinstance);
 	       newpoints[5] = newpoints[0];
 	       lastmindist = mindist;
 	       mindist = LONG_MAX;
@@ -1769,9 +1768,9 @@ void UDrawBBox()
 {
    XPoint	origin;
    XPoint	worig, wcorn, corner;
-   objinstptr	bbinst = areastruct.topinstance;
+   objinstptr	bbinst = areawin->topinstance;
 
-   if ((!areastruct.bboxon) || (checkforbbox(topobject) != NULL)) return;
+   if ((!areawin->bboxon) || (checkforbbox(topobject) != NULL)) return;
 
    origin = bbinst->bbox.lowerleft;
    corner.x = origin.x + bbinst->bbox.width;
@@ -1783,14 +1782,14 @@ void UDrawBBox()
    user_to_window(origin, &worig);
    user_to_window(corner, &wcorn);
 
-   SetForeground(dpy, areastruct.gc, BBOXCOLOR);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
+   SetForeground(dpy, areawin->gc, BBOXCOLOR);
+   DrawLine(dpy, areawin->window, areawin->gc, worig.x, worig.y,
 		worig.x, wcorn.y);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
+   DrawLine(dpy, areawin->window, areawin->gc, worig.x, wcorn.y,
 		wcorn.x, wcorn.y);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
+   DrawLine(dpy, areawin->window, areawin->gc, wcorn.x, wcorn.y,
 		wcorn.x, worig.y);
-   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
+   DrawLine(dpy, areawin->window, areawin->gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
 
@@ -1805,27 +1804,27 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
    float        tmpwidth;
    short	minwidth;
 
-   tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth * width);
+   tmpwidth = UTopTransScale(xobjs.pagelist[areawin->page]->wirewidth * width);
    minwidth = max(1, (short)tmpwidth);
 
    if (style & FILLED || (!(style & FILLED) && style & OPAQUE)) {
       if ((style & FILLSOLID) == FILLSOLID)
-         SetFillStyle(dpy, areastruct.gc, FillSolid);
+         SetFillStyle(dpy, areawin->gc, FillSolid);
       else if (!(style & FILLED)) {
-         SetFillStyle(dpy, areastruct.gc, FillOpaqueStippled); 
-	 SetStipple(dpy, areastruct.gc, 7);
+         SetFillStyle(dpy, areawin->gc, FillOpaqueStippled); 
+	 SetStipple(dpy, areawin->gc, 7);
       }
       else {
 	 if (style & OPAQUE)
-            SetFillStyle(dpy, areastruct.gc, FillOpaqueStippled);
+            SetFillStyle(dpy, areawin->gc, FillOpaqueStippled);
 	 else
-            SetFillStyle(dpy, areastruct.gc, FillStippled);
-         SetStipple(dpy, areastruct.gc, ((style & FILLSOLID) >> 5));
+            SetFillStyle(dpy, areawin->gc, FillStippled);
+         SetStipple(dpy, areawin->gc, ((style & FILLSOLID) >> 5));
       }
-      FillPolygon(dpy, areastruct.areawin, areastruct.gc, pathlist, number, Nonconvex,
+      FillPolygon(dpy, areawin->window, areawin->gc, pathlist, number, Nonconvex,
 		CoordModeOrigin);
       /* return to original state */
-      SetFillStyle(dpy, areastruct.gc, FillSolid);
+      SetFillStyle(dpy, areawin->gc, FillSolid);
    }
    if (!(style & NOBORDER)) {
       /* set up dots or dashes */
@@ -1833,19 +1832,19 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
       else if (style & DOTTED) solidpart = (char)minwidth;
       sprintf(dashstring, "%c%c", solidpart, (char)(4 * minwidth));
       if (style & (DASHED | DOTTED)) {
-         SetDashes(dpy, areastruct.gc, 0, dashstring, 2);
-         SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineOnOffDash,
+         SetDashes(dpy, areawin->gc, 0, dashstring, 2);
+         SetLineAttributes(dpy, areawin->gc, tmpwidth, LineOnOffDash,
 		CapButt, JoinBevel);
       }
       else
-         SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid,
+         SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid,
 		CapRound, JoinBevel);
 
       /* draw the spline and close off if so specified */
-      DrawLines(dpy, areastruct.areawin, areastruct.gc, pathlist,
+      DrawLines(dpy, areawin->window, areawin->gc, pathlist,
 		number, CoordModeOrigin);
       if (!(style & UNCLOSED))
-         DrawLine(dpy, areastruct.areawin, areastruct.gc, pathlist[0].x,
+         DrawLine(dpy, areawin->window, areawin->gc, pathlist[0].x,
 		pathlist[0].y, pathlist[number - 1].x, pathlist[number - 1].y);
    }
 }
@@ -1957,7 +1956,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
    int		thispart;
-   XPoint 	bboxin, bboxout[2];
+   XPoint 	bboxin[2], bboxout[2];
    u_char	xm, ym;
    objectptr	theobject = theinstance->thisobject;
 
@@ -1974,15 +1973,18 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 
    /* do a quick test for intersection with the display window */
 
-   UTransformbyCTM(DCTM, &(theobject->bbox.lowerleft), &(bboxout[0]), 1);
-   bboxin.x = theobject->bbox.lowerleft.x + theobject->bbox.width;
-   bboxin.y = theobject->bbox.lowerleft.y + theobject->bbox.height; 
-   UTransformbyCTM(DCTM, &bboxin, &(bboxout[1]), 1);
+   bboxin[0].x = theobject->bbox.lowerleft.x;
+   bboxin[0].y = theobject->bbox.lowerleft.y;
+   bboxin[1].x = theobject->bbox.lowerleft.x + theobject->bbox.width;
+   bboxin[1].y = theobject->bbox.lowerleft.y + theobject->bbox.height; 
+   if (level == 0)
+      extendschembbox(theinstance, &(bboxin[0]), &(bboxin[1]));
+   UTransformbyCTM(DCTM, bboxin, bboxout, 2);
 
    xm = (bboxout[0].x < bboxout[1].x) ? 0 : 1;  
    ym = (bboxout[0].y < bboxout[1].y) ? 0 : 1;  
 
-   if (bboxout[xm].x < areastruct.width && bboxout[ym].y < areastruct.height &&
+   if (bboxout[xm].x < areawin->width && bboxout[ym].y < areawin->height &&
        bboxout[1 - xm].x > 0 && bboxout[1 - ym].y > 0) {       
 
      /* make parameter substitutions */
@@ -1990,8 +1992,8 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 
      /* draw all of the elements */
    
-     tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
-     SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid, CapRound,
+     tmpwidth = UTopTransScale(xobjs.pagelist[areawin->page]->wirewidth);
+     SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid, CapRound,
 		JoinBevel);
 
      /* guard against plist being regenerated during a redraw by the	*/
@@ -2034,11 +2036,11 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 	     break;
    
           case(OBJINST):
-	     if (areastruct.editinplace && stack && (TOOBJINST(areagen)
-			== areastruct.topinstance)) {
-		/* If stack matches areastruct.stack, then don't draw */
+	     if (areawin->editinplace && stack && (TOOBJINST(areagen)
+			== areawin->topinstance)) {
+		/* If stack matches areawin->stack, then don't draw */
 		/* because it would be redundant.		 */
-		pushlistptr alist = *stack, blist = areastruct.stack;
+		pushlistptr alist = *stack, blist = areawin->stack;
 		while (alist && blist) {
 		   if (alist->thisinst != blist->thisinst) break;
 		   alist = alist->next;
@@ -2054,7 +2056,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 			(TOLABEL(areagen)->justify & PINVISIBLE))
              UDrawString(TOLABEL(areagen), curcolor, theinstance);
 	     else if (level == 1 && TOLABEL(areagen)->pin &&
-			TOLABEL(areagen)->pin != INFO && areastruct.pinpointon)
+			TOLABEL(areagen)->pin != INFO && areawin->pinpointon)
 		UDrawXDown(TOLABEL(areagen));
 	     break;
 
@@ -2166,15 +2168,19 @@ void TopDoLatex()
    Boolean checklatex = FALSE;
    char filename[50];
 
-   UDoLatex(areastruct.topinstance, 0, NULL, 1.0, 0, 0, &checklatex);
+   UDoLatex(areawin->topinstance, 0, NULL, 1.0, 0, 0, &checklatex);
 
    if (checklatex == FALSE) return;	/* No LaTeX labels to write */
 
-   sprintf(filename, "%s.tex", xobjs.pagelist[areastruct.page]->filename);
+   if (xobjs.pagelist[areawin->page]->filename)
+      sprintf(filename, "%s.tex", xobjs.pagelist[areawin->page]->filename);
+   else
+      sprintf(filename, "%s.tex",
+		xobjs.pagelist[areawin->page]->pageinst->thisobject->name);
    f = fopen(filename, "w");
 
    fprintf(f, "%% XCircuit output \"%s\" for LaTeX input from %s.ps\n",
-		filename, xobjs.pagelist[areastruct.page]->filename);
+		filename, xobjs.pagelist[areawin->page]->filename);
    fprintf(f, "\\def\\putbox#1#2#3{\\makebox[0in][l]{\\makebox[#1][l]{}"
 		"\\raisebox{\\baselineskip}[0in][0in]"
 		"{\\raisebox{#2}[0in][0in]{#3}}}}\n");
@@ -2183,14 +2189,20 @@ void TopDoLatex()
    fprintf(f, "\\def\\topbox#1{\\raisebox{-\\baselineskip}[0in][0in]{#1}}\n");
    fprintf(f, "\\def\\midbox#1{\\raisebox{-0.5\\baselineskip}[0in][0in]{#1}}\n");
    fprintf(f, "\\begin{flushleft}\n");
-   fprintf(f, "\\epsfig{file=%s.ps}\\\\\n",
-		xobjs.pagelist[areastruct.page]->filename);
+   
+   /* Handle cases where the file might have a ".eps" extension.	*/
+   /* Thanks to Graham Sheward for pointing this out.			*/
+   filename[strlen(filename) - 4] = '\0';
+   if (strchr(filename, '.') == NULL)
+       sprintf(filename + strlen(filename), ".ps");
 
-   psscale = getpsscale(xobjs.pagelist[areastruct.page]->outscale,
-		areastruct.page);
+   fprintf(f, "\\epsfig{file=%s}\\\\\n", filename);
 
-   width = toplevelwidth(areastruct.topinstance, &origin.x);
-   height = toplevelheight(areastruct.topinstance, &origin.y);
+   psscale = getpsscale(xobjs.pagelist[areawin->page]->outscale,
+		areawin->page);
+
+   width = toplevelwidth(areawin->topinstance, &origin.x);
+   height = toplevelheight(areawin->topinstance, &origin.y);
 
    tx = (int)(72 / psscale) - origin.x,
    ty = (int)(72 / psscale) - origin.y;
@@ -2199,7 +2211,7 @@ void TopDoLatex()
 
    UPushCTM();		/* Save current state */
    UResetCTM(DCTM);	/* Set to identity matrix */
-   UDoLatex(areastruct.topinstance, 0, f, psscale, tx, ty, NULL);
+   UDoLatex(areawin->topinstance, 0, f, psscale, tx, ty, NULL);
    UPopCTM();		/* Restore state */
 
    fprintf(f, "\\end{flushleft}\n");
