@@ -37,12 +37,11 @@
 /* Global Variable definitions						   */
 /*-------------------------------------------------------------------------*/
 
-extern short   eventmode;	/* keep track of the mode the screen is in */
 extern Display	*dpy;   /* Works well to make this globally accessible */
 extern int *appcolors;
 extern Cursor	appcursors[NUM_CURSORS];
 extern Globaldata xobjs;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern char _STR[150];
 extern short fontcount;
 extern fontinfo *fonts;
@@ -102,19 +101,19 @@ void catreturn()
 /*   page number of page to the right.  		*/
 /*------------------------------------------------------*/
 
-int pageposition(short libmode, XButtonEvent *event, int mode)
+int pageposition(short libmode, int x, int y, int mode)
 {
    int xin, yin, bpage, pages;
    int gxsize, gysize, xdel, ydel;
 
    pages = (libmode == PAGELIB) ? xobjs.pages : xobjs.numlibs;
    computespacing(libmode, &gxsize, &gysize, &xdel, &ydel);
-   window_to_user(event->x, event->y, &areastruct.save);
+   window_to_user(x, y, &areawin->save);
 
    if (mode == 0) {	/* On-page */
-      if (areastruct.save.x >= 0 && areastruct.save.y <= 0) {
-         xin = areastruct.save.x / xdel;
-         yin = areastruct.save.y / ydel; 
+      if (areawin->save.x >= 0 && areawin->save.y <= 0) {
+         xin = areawin->save.x / xdel;
+         yin = areawin->save.y / ydel; 
          if (xin < gxsize && yin > -gysize) {
             bpage = (xin % gxsize) - (yin * gxsize);
             if (bpage < pages)
@@ -124,10 +123,10 @@ int pageposition(short libmode, XButtonEvent *event, int mode)
       return -1;
    }
    else {		/* Between-pages */
-      xin = (areastruct.save.x + (xdel >> 1)) / xdel;
+      xin = (areawin->save.x + (xdel >> 1)) / xdel;
       if (xin > gxsize) xin = gxsize;
       if (xin < 0) xin = 0;
-      yin = areastruct.save.y  / ydel; 
+      yin = areawin->save.y  / ydel; 
       if (yin > 0) yin = 0;
       if (yin < -gysize) yin = -gysize;
       bpage = (xin % (gxsize + 1)) + 1 - (yin * gxsize);
@@ -151,8 +150,10 @@ short pagelinks(int page)
    for (i = 0; i < xobjs.pages; i++)
       if (xobjs.pagelist[i]->pageinst != NULL)
 	 if (xobjs.pagelist[i]->pageinst->thisobject->parts > 0)
-	    if ((i == page) || (!strcmp(xobjs.pagelist[i]->filename,
-			xobjs.pagelist[page]->filename)))
+	    if ((i == page) || (xobjs.pagelist[i]->filename &&
+			xobjs.pagelist[page]->filename &&
+			(!strcmp(xobjs.pagelist[i]->filename,
+			xobjs.pagelist[page]->filename))))
 	       count++;
 
    return count;
@@ -212,9 +213,11 @@ short *pagetotals(int page, short mode)
 		  (*(counts + i)) = 1;
 	       else
 	       {
-	          if ((i == page) || (!strcmp(xobjs.pagelist[i]->filename,
-			xobjs.pagelist[page]->filename)))
-	             if (*(counts + i) == 0)
+	          if ((i == page) || (xobjs.pagelist[i]->filename
+			&& xobjs.pagelist[page]->filename
+			&& (!strcmp(xobjs.pagelist[i]->filename,
+			xobjs.pagelist[page]->filename))))
+		     if ((mode == INDEPENDENT) || (*(counts + i) == 0))
 		        (*(icount + i))++;
 	       }
 	    }
@@ -297,52 +300,52 @@ int check_library(char *libname)
 /* ButtonPress handler during page catalog viewing mode */
 /*------------------------------------------------------*/
 
-void pagecatbutton(XButtonEvent *event)
+void pagecat_op(int op, int x, int y)
 {
    int bpage;
    short mode;
 
    for (mode = 0; mode < LIBRARY; mode++) {
-      if (areastruct.topinstance == xobjs.libtop[mode]) break;
+      if (areawin->topinstance == xobjs.libtop[mode]) break;
    }
    if (mode == LIBRARY) return;  /* Something went wrong if this happens */
 
-   if (event->button == Button1 || event->button == Button2) {
-      if ((bpage = pageposition(mode, event, 0)) >= 0) {
+   if (op != XCF_Cancel) {
+      if ((bpage = pageposition(mode, x, y, 0)) >= 0) {
 		  
 	 if (eventmode == ASSOC_MODE) {
 	    if (mode == PAGELIB) {
 	       /* using changepage() allows use of new page for schematic */
 	       changepage(bpage);
 	       /* associate the new schematic */
-	       schemassoc(topobject, areastruct.stack->thisinst->thisobject);
+	       schemassoc(topobject, areawin->stack->thisinst->thisobject);
 	       /* pop back to calling (symbol) page */
 	       catreturn();
-	       eventmode = PRESS_MODE;
+	       eventmode = NORMAL_MODE;
 	    }
 	    else {
+	       areawin->lastlibrary = bpage;
 	       startcatalog(NULL, (pointertype)(LIBRARY + bpage), NULL);
 	    }
 	    return;
          }
-	 else if (event->button == Button1) {
+	 else if (op == XCF_Select) {
+	    if (mode == PAGELIB)    /* No such method for LIBLIB is defined. */
+	       select_add_element(OBJINST);
+	 }
+	 else if ((op == XCF_Library_Pop) || (op == XCF_Finish)) {
 
 	    /* like catreturn(), but don't actually go to the popped page */
 	    unselect_all();
 	    eventmode = NORMAL_MODE;
 	    if (mode == PAGELIB) {
 	       newpage(bpage);
-      	       eventmode = PRESS_MODE;
 	    }
 	    else {
 	       startcatalog(NULL, (pointertype)(LIBRARY + bpage), NULL);
 	    }
 	    return;
 	 }
-	 else {  /* Button2 */
-	    if (mode == PAGELIB)    /* No such method for LIBLIB is defined. */
-	       select_add_element(OBJINST);
-	 }
       }
    }
    else {
@@ -418,8 +421,8 @@ void computespacing(short mode, int *gxsize, int *gysize, int *xdel, int *ydel)
 
    /* 0.5 is the default vscale;  g#size is the number of pages per line */
 
-   *xdel = areastruct.width / (0.5 * (*gxsize));
-   *ydel = areastruct.height / (0.5 * (*gysize));
+   *xdel = areawin->width / (0.5 * (*gxsize));
+   *ydel = areawin->height / (0.5 * (*gysize));
 }
 
 /*-------------------------------------------------------------------*/
@@ -566,29 +569,29 @@ void updatepagelib(short mode, short tpage)
 /* Rearrange pages	*/
 /*----------------------*/
 
-void pagecatmove(XKeyEvent *event)
+void pagecatmove(int x, int y)
 {
    int bpage;
    objinstptr exchobj;
    Pagedata *ipage, **testpage, **tpage2;
 
-   if (areastruct.selects == 0) return;
-   else if (areastruct.selects > 2) {
+   if (areawin->selects == 0) return;
+   else if (areawin->selects > 2) {
       Wprintf("Select maximum of two objects.");
       return;
    }
 
    /* Get the page corresponding to the first selected object */
 
-   exchobj = SELTOOBJINST(areastruct.selectlist);
+   exchobj = SELTOOBJINST(areawin->selectlist);
    for (testpage = xobjs.pagelist; testpage < xobjs.pagelist + xobjs.pages; testpage++)
       if (*testpage != NULL && (*testpage)->pageinst == exchobj)
 	 break;
 
    /* If two objects are selected, then exchange their order */
 
-   if (areastruct.selects == 2) {
-      exchobj = SELTOOBJINST(areastruct.selectlist + 1);
+   if (areawin->selects == 2) {
+      exchobj = SELTOOBJINST(areawin->selectlist + 1);
       for (tpage2 = xobjs.pagelist; tpage2 < xobjs.pagelist + xobjs.pages; tpage2++)
 	 if (*tpage2 != NULL && (*tpage2)->pageinst == exchobj)
 	    break;
@@ -600,7 +603,7 @@ void pagecatmove(XKeyEvent *event)
 
    /* If one object selected; find place to put from cursor position */
 
-   else if ((bpage = pageposition(PAGELIB, (XButtonEvent *)event, 1)) >= 0) {
+   else if ((bpage = pageposition(PAGELIB, x, y, 1)) >= 0) {
       int k, epage;
       Pagedata *eptr;
 
@@ -646,7 +649,7 @@ void composelib(short mode)
    labelptr *drawname;
    objectptr libobj, libpage = xobjs.libtop[mode]->thisobject;
    liblistptr spec;
-   int xpos = 0, ypos = areastruct.height << 1;
+   int xpos = 0, ypos = areawin->height << 1;
    int nypos = 220, nxpos;
    short fval;
    short llx, lly, width, height;
@@ -703,8 +706,8 @@ void composelib(short mode)
       totalarea += (width * height);
    }
 
-   scale = (double)totalarea / (double)(areastruct.width * areastruct.height);
-   targetwidth = (int)(sqrt(scale) * (double)areastruct.width);
+   scale = (double)totalarea / (double)(areawin->width * areawin->height);
+   targetwidth = (int)(sqrt(scale) * (double)areawin->width);
 
    /* generate the list of object instances and their labels */
 
@@ -727,7 +730,7 @@ void composelib(short mode)
       /* Determine the area needed on the page to draw the object */
 
       nxpos = xpos + ((width > 170) ? width + 30 : 200);
-      /* if ((nxpos > (areastruct.width << 1)) && (xpos > 0)) { */
+      /* if ((nxpos > (areawin->width << 1)) && (xpos > 0)) { */
       if ((nxpos > targetwidth) && (xpos > 0)) {
 	 nxpos -= xpos; 
 	 xpos = 0;
@@ -840,14 +843,14 @@ void catvirtualcopy()
    short i, *newselect;
    objinstptr libobj, libinst;
 
-   if (areastruct.selects == 0) return;
+   if (areawin->selects == 0) return;
    else if ((i = is_library(topobject)) < 0) return;
 
    /* Check for existance of parameters in the object for each */
    /* selected instance */
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
       libobj = SELTOOBJINST(newselect);
 
       if (libobj->thisobject->params == NULL) {
@@ -878,18 +881,17 @@ void cathide()
    objectptr *compobj;
    objinstptr libobj;
 
-   if (areastruct.selects == 0) return;
+   if (areawin->selects == 0) return;
 
    /* Can only hide objects which are instances in other objects; */
    /* Otherwise, object would be "lost".			  */
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
       libobj = SELTOOBJINST(newselect);
 
       if (finddepend(libobj, &compobj) == 0) {
-	 sprintf(_STR, "Cannot hide: no dependencies");
-	 Wprintf(_STR);
+	 Wprintf("Cannot hide: no dependencies");
       }
       else { 		/* All's clear to hide. */
 	 libobj->thisobject->hidden = True;
@@ -916,15 +918,15 @@ void catdelete()
    liblistptr ilist, llist;
    objectptr *libpage, *compobj, *tlib, *slib;
 
-   if (areastruct.selects == 0) return;
+   if (areawin->selects == 0) return;
 
    if ((i = is_library(topobject)) >= 0) {
       libpage = xobjs.userlibs[i].library;
       libpobjs = &xobjs.userlibs[i].number;
    }
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
       libobj = SELTOOBJINST(newselect);
 
       /* If this is just a "virtual copy", simply remove it from the list */
@@ -949,8 +951,7 @@ void catdelete()
       /* or if the object is used on a page.				   */
 
       if (finddepend(libobj, &compobj)) {
-	 sprintf(_STR, "Cannot delete: dependency in \"%s\"", (*compobj)->name);
-	 Wprintf(_STR);
+	 Wprintf("Cannot delete: dependency in \"%s\"", (*compobj)->name);
       }
       else { 		/* All's clear to delete.		     	   */
 
@@ -1081,7 +1082,7 @@ void linkedlistinsertafter(liblistptr *spec, int o1, int o2)
 /* Rearrange objects in the library			*/
 /*------------------------------------------------------*/
 
-void catmove(XKeyEvent *event)
+void catmove(int x, int y)
 {
    int i, j, s1, s2, ocentx, ocenty, rangex, rangey;
    liblistptr spec;
@@ -1090,17 +1091,17 @@ void catmove(XKeyEvent *event)
    /* make catmove() a wrapper for pagecatmove() */
 
    if ((i = is_library(topobject)) < 0) {
-      pagecatmove(event);
+      pagecatmove(x, y);
       return;
    }
 
-   if (areastruct.selects == 0) return;
-   else if (areastruct.selects > 2) {
+   if (areawin->selects == 0) return;
+   else if (areawin->selects > 2) {
       Wprintf("Select maximum of two objects.");
       return;
    }
 
-   exchobj = SELTOOBJINST(areastruct.selectlist);
+   exchobj = SELTOOBJINST(areawin->selectlist);
    s1 = -1;
    for (j = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
 		spec = spec->next, j++) {
@@ -1112,8 +1113,8 @@ void catmove(XKeyEvent *event)
 
    /* If two objects are selected, then exchange their order */
 
-   if (areastruct.selects == 2) {
-      lobj = SELTOOBJINST(areastruct.selectlist + 1);
+   if (areawin->selects == 2) {
+      lobj = SELTOOBJINST(areawin->selectlist + 1);
 
       s2 = -1;
       for (j = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
@@ -1128,7 +1129,7 @@ void catmove(XKeyEvent *event)
    }
    else {	/* one object selected; find place to put from cursor position */
 
-      window_to_user(event->x, event->y, &areastruct.save);
+      window_to_user(x, y, &areawin->save);
 
       s2 = -1;
       for (j = 0, spec = xobjs.userlibs[i].instlist; spec != NULL;
@@ -1142,18 +1143,18 @@ void catmove(XKeyEvent *event)
 	 rangey = (lobj->bbox.height > 200) ? 
 	        (lobj->bbox.height >> 1) : 100;
 
-	 if ((areastruct.save.y < ocenty + rangey) && (areastruct.save.y 
+	 if ((areawin->save.y < ocenty + rangey) && (areawin->save.y 
 	         > ocenty - rangey)) {
 	    s2 = j - 1;
-	    if (areastruct.save.x < ocentx) break;
+	    if (areawin->save.x < ocentx) break;
 	    else s2 = j;
 	 }
       }
       if ((s2 == -1) && (spec == NULL)) {
-	 if (areastruct.save.y <
+	 if (areawin->save.y <
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.lowerleft.y)
 	    s2 = j - 1;
-	 else if (areastruct.save.y <=
+	 else if (areawin->save.y <=
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.lowerleft.y +
 		xobjs.libtop[i + LIBRARY]->thisobject->bbox.height) {
 	    unselect_all();	
@@ -1184,8 +1185,8 @@ void copycat()
    oparamptr ops, newops;
    int i, libnum = USERLIB - LIBRARY ;
 
-   for (newselect = areastruct.selectlist; newselect < areastruct.selectlist
-	  + areastruct.selects; newselect++) {
+   for (newselect = areawin->selectlist; newselect < areawin->selectlist
+	  + areawin->selects; newselect++) {
 
       libobj = SELTOOBJINST(newselect);
       oldobj = libobj->thisobject;
@@ -1252,7 +1253,6 @@ void copycat()
       (*newobj)->labels = NULL;
       (*newobj)->valid = False;
       (*newobj)->traversed = False;
-      (*newobj)->devname = NULL;
       (*newobj)->hidden = False;
 
       /* copy over all the elements of the original object */
@@ -1348,7 +1348,7 @@ void copycat()
    composelib(USERLIB);
    unselect_all();
 
-   if (areastruct.topinstance == xobjs.libtop[USERLIB])
+   if (areawin->topinstance == xobjs.libtop[USERLIB])
       drawarea(NULL, NULL, NULL);
    else startcatalog(NULL, (pointertype)USERLIB, NULL);
 }
@@ -1357,7 +1357,7 @@ void copycat()
 /* ButtonPress handler during normal catalog viewing mode */
 /*--------------------------------------------------------*/
 
-void catbutton(u_int mode, XButtonEvent *event)
+void catalog_op(int op, int x, int y)
 {
    short *newselect;
    objinstptr *newobject, *libobj;
@@ -1365,16 +1365,22 @@ void catbutton(u_int mode, XButtonEvent *event)
    short ocentx, ocenty, rangex, rangey, xdiff, ydiff, flag = 0;
    XPoint oldpos;
 
-   /* Make catbutton() a wrapper for pagecatbutton() */
+   /* Make catalog_op() a wrapper for pagecat_op() */
 
    if (is_library(topobject) < 0) {
-      pagecatbutton(event);
+      pagecat_op(op, x, y);
       return;
    }
 
-   if (event->button == Button1 || event->button == Button2) {
+   /* If XCF_Cancel was invoked, return without a selection. */
 
-      window_to_user(event->x, event->y, &areastruct.save);
+   if (op == XCF_Cancel) {
+      eventmode = NORMAL_MODE;
+      catreturn();
+   }
+   else {
+
+      window_to_user(x, y, &areawin->save);
 	
       for (libobj = (objinstptr *)topobject->plist; libobj <
 		(objinstptr *)topobject->plist + topobject->parts; libobj++) {
@@ -1390,52 +1396,52 @@ void catbutton(u_int mode, XButtonEvent *event)
 	    rangey = ((*libobj)->bbox.height > 200) ? 
 	        ((*libobj)->bbox.height >> 1) : 100;
 
-	    if (areastruct.save.x > ocentx - rangex && areastruct.save.x <
-		   ocentx + rangex && areastruct.save.y < ocenty + rangey
-		   && areastruct.save.y > ocenty - rangey) {
+	    if (areawin->save.x > ocentx - rangex && areawin->save.x <
+		   ocentx + rangex && areawin->save.y < ocenty + rangey
+		   && areawin->save.y > ocenty - rangey) {
 
 	       /* setup to move object around and draw the selected object */
 
 	       if (eventmode == ASSOC_MODE) {
 
 	          /* revert to old page */
-	          areastruct.topinstance = (areastruct.stack == NULL) ?
-			   xobjs.pagelist[areastruct.page]->pageinst
-			   : areastruct.stack->thisinst;
+	          areawin->topinstance = (areawin->stack == NULL) ?
+			   xobjs.pagelist[areawin->page]->pageinst
+			   : areawin->stack->thisinst;
 		  /* associate the new object */
 		  schemassoc(topobject, (*libobj)->thisobject); 
    	          setpage(TRUE);
 		  catreturn();
-		  eventmode = PRESS_MODE;
+		  eventmode = NORMAL_MODE;
 	       }
-	       else if (event->button == Button1) {
+	       else if ((op == XCF_Library_Pop) || (op == XCF_Library_Copy)) {
 
 	          /* revert to old page */
 
-	          areastruct.topinstance = (areastruct.stack == NULL) ?
-			   xobjs.pagelist[areastruct.page]->pageinst
-			   : areastruct.stack->thisinst;
+	          areawin->topinstance = (areawin->stack == NULL) ?
+			   xobjs.pagelist[areawin->page]->pageinst
+			   : areawin->stack->thisinst;
 
 	          /* retrieve drawing window state and set position of object to
 	             be correct in reference to that window */
 
-	          snap(event->x, event->y, &oldpos);
+	          snap(x, y, &oldpos);
 
    	          setpage(FALSE);
 	    
-	          snap(event->x, event->y, &areastruct.save);
-	          xdiff = areastruct.save.x - oldpos.x;
-	          ydiff = areastruct.save.y - oldpos.y;
+	          snap(x, y, &areawin->save);
+	          xdiff = areawin->save.x - oldpos.x;
+	          ydiff = areawin->save.y - oldpos.y;
 
                   /* collect all of the selected items */
 
-	          for (newselect = areastruct.selectlist; newselect <
-		     areastruct.selectlist + areastruct.selects; newselect++) {
+	          for (newselect = areawin->selectlist; newselect <
+		     areawin->selectlist + areawin->selects; newselect++) {
 		     NEW_OBJINST(newobject, topobject);
 		     topobject->parts++;
 		     instcopy(*newobject, TOOBJINST(libpage->plist + *newselect));
 		     /* color should be recast as current color */
-		     (*newobject)->color = areastruct.color;
+		     (*newobject)->color = areawin->color;
 		     /* position modified by (xdiff, ydiff) */
 		     (*newobject)->position.x += xdiff;
 		     (*newobject)->position.y += ydiff;
@@ -1452,9 +1458,9 @@ void catbutton(u_int mode, XButtonEvent *event)
 		     NEW_OBJINST(newobject, topobject);
                      topobject->parts++;
 		     instcopy(*newobject, *libobj);
-		     (*newobject)->color = areastruct.color;
-                     (*newobject)->position.x = areastruct.save.x;
-	             (*newobject)->position.y = areastruct.save.y; 
+		     (*newobject)->color = areawin->color;
+                     (*newobject)->position.x = areawin->save.x;
+	             (*newobject)->position.y = areawin->save.y; 
 
 	             /* add this object to the list of selected items */
 
@@ -1462,42 +1468,43 @@ void catbutton(u_int mode, XButtonEvent *event)
                      *newselect = (short)(newobject - (objinstptr *)topobject->plist);
 
 	          }
-	          if ((void *)mode == (void *)(1)) {
+	          if (op == XCF_Library_Copy) {
 
-		     /* key "c" pressed for "copy" */
+		     /* Key "c" pressed for "copy" (default binding) */
 
-                     XDefineCursor(dpy, areastruct.areawin, COPYCURSOR);
-                     eventmode = COPY2_MODE;
+                     XDefineCursor(dpy, areawin->window, COPYCURSOR);
+                     eventmode = COPY_MODE;
 #ifndef TCL_WRAPPER
-                     xcAddEventHandler(areastruct.area, PointerMotionMask, False,
-			  (xcEventHandler)drag, NULL);
+                     xcAddEventHandler(areawin->area, PointerMotionMask, False,
+			  (xcEventHandler)xlib_drag, NULL);
 #endif
 	          }
 	          else {
-                     eventmode = PRESS_MODE;
+                     eventmode = MOVE_MODE;
+	             areawin->editcycle = 1;
 	          }
 #ifdef TCL_WRAPPER
 		  /* fprintf(stderr, "Creating event handler for xctk_drag: "); */
 		  /* printeventmode();		*/
-                  Tk_CreateEventHandler(areastruct.area, PointerMotionMask,
+                  Tk_CreateEventHandler(areawin->area, PointerMotionMask,
 			(Tk_EventProc *)xctk_drag, NULL);
 #endif
-	 	  register_for_undo(XCF_Library_Pop, UNDO_MORE, areastruct.topinstance,
-				areastruct.selectlist, areastruct.selects);
+	 	  register_for_undo(XCF_Library_Pop, UNDO_MORE, areawin->topinstance,
+				areawin->selectlist, areawin->selects);
                   catreturn();
                }
 
-               /* If button2 was pressed, select and stay in the catalog.  */
+               /* Select the closest element and stay in the catalog.	   */
 	       /* Could just do "select_element" here, but would not cover */
 	       /* the entire area in the directory surrounding the object. */
 
-	       else {
+	       else if (op == XCF_Select) {
 		  short newinst = (short)(libobj - (objinstptr *)topobject->plist);
 		  /* (ignore this object if it is already in the list of selects) */
-		  for (newselect = areastruct.selectlist; newselect <
-			areastruct.selectlist + areastruct.selects; newselect++)
+		  for (newselect = areawin->selectlist; newselect <
+			areawin->selectlist + areawin->selects; newselect++)
 		     if (*newselect == newinst) break;
-		  if (newselect == areastruct.selectlist + areastruct.selects) {
+		  if (newselect == areawin->selectlist + areawin->selects) {
 		     newselect = allocselect();
 		     *newselect = newinst;
 		     XcSetFunction(GXcopy);
@@ -1510,12 +1517,6 @@ void catbutton(u_int mode, XButtonEvent *event)
 	 }
       }
    }
-   /* If button3 was pressed, return without a selection. */
-
-   else {
-      eventmode = NORMAL_MODE;
-      catreturn();
-   }
 }
 
 /*------------------------------*/
@@ -1527,8 +1528,8 @@ void changecat()
    int i, j;
 
    if ((i = is_library(topobject)) < 0) {
-      if (areastruct.lastlibrary >= xobjs.numlibs) areastruct.lastlibrary = 0;
-      j = areastruct.lastlibrary;
+      if (areawin->lastlibrary >= xobjs.numlibs) areawin->lastlibrary = 0;
+      j = areawin->lastlibrary;
       eventmode = CATALOG_MODE;
    }
    else {
@@ -1537,7 +1538,7 @@ void changecat()
 	 Wprintf("This is the only library.");
 	 return;
       }
-      areastruct.lastlibrary = j;
+      areawin->lastlibrary = j;
    }
    startcatalog(NULL, (pointertype)(j + LIBRARY), NULL);
 }
@@ -1551,20 +1552,20 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
    if (xobjs.libtop == NULL) return;	/* No libraries defined */
 
    if ((xobjs.libtop[libmod]->thisobject == NULL) ||
-		(areastruct.topinstance == xobjs.libtop[libmod])) return;
+		(areawin->topinstance == xobjs.libtop[libmod])) return;
 
    if (libmod == FONTLIB) {
-      XDefineCursor (dpy, areastruct.areawin, CROSS);
-      if (eventmode == TEXT2_MODE)
+      XDefineCursor (dpy, areawin->window, DEFAULTCURSOR);
+      if (eventmode == TEXT_MODE)
          eventmode = FONTCAT_MODE;
       else
-         eventmode = FONTCAT2_MODE;
+         eventmode = EFONTCAT_MODE;
    }
    else if (eventmode == ASSOC_MODE) {
-      XDefineCursor (dpy, areastruct.areawin, CROSS);
+      XDefineCursor (dpy, areawin->window, DEFAULTCURSOR);
    }
    else if (libmod == PAGELIB || libmod == LIBLIB) {
-      XDefineCursor (dpy, areastruct.areawin, CROSS);
+      XDefineCursor (dpy, areawin->window, DEFAULTCURSOR);
       eventmode = CATALOG_MODE;
    }
    else
@@ -1576,16 +1577,17 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
 
 
    if (!(((is_library(topobject) >= 0)
-		|| (areastruct.topinstance == xobjs.libtop[LIBLIB])
-		|| (areastruct.topinstance == xobjs.libtop[PAGELIB]))
+		|| (areawin->topinstance == xobjs.libtop[LIBLIB])
+		|| (areawin->topinstance == xobjs.libtop[PAGELIB]))
 		&& libmod >= PAGELIB)) {
-      push_stack(&areastruct.stack, areastruct.topinstance);
+      push_stack(&areawin->stack, areawin->topinstance);
    }
 
    /* set library as new object */
 
-   areastruct.topinstance = xobjs.libtop[libmod];
+   areawin->topinstance = xobjs.libtop[libmod];
    setpage(TRUE);
+   invalidate_graphics(topobject);
 
    /* draw the new screen */
 
