@@ -478,7 +478,7 @@ extern void setkern(xcWidget, stringpart *);
 extern void getdscale(xcWidget, caddr_t, caddr_t);
 extern void setdscale(xcWidget, XPoint *);
 extern labelptr gettextsize(float **);
-extern void gettsize(xcWidget, caddr_t, caddr_t);
+/* Xt */ extern void gettsize(xcWidget, caddr_t, caddr_t);
 extern void setosize(xcWidget, objinstptr);
 extern void getosize(xcWidget, caddr_t, caddr_t);
 extern void getwirewidth(xcWidget, caddr_t, caddr_t);
@@ -497,10 +497,10 @@ extern void toolcursor(int);
 extern void promptparam(xcWidget, caddr_t, caddr_t);
 #endif
 extern void stringparam(xcWidget, caddr_t, caddr_t);
-extern void toggle(xcWidget, Boolean *, caddr_t);
+extern void toggle(xcWidget, pointertype, Boolean *);
 extern int setelementstyle(xcWidget, u_short, u_short);
 extern void setcolorscheme(Boolean);
-extern void inversecolor(xcWidget, Boolean *, caddr_t);
+extern void inversecolor(xcWidget, pointertype, caddr_t);
 extern void togglegrid(u_short);
 extern float parseunits(char *);
 extern void setgridtype(char *);
@@ -839,8 +839,13 @@ extern void fontcat_op(int, int, int);
 
 /* from xcircuit.c: */
 
+extern void Wprintf(char *, ...);
+extern void W1printf(char *, ...);
+extern void W2printf(char *, ...);
+extern void W3printf(char *, ...);
+
+extern XCWindowData *create_new_window();
 extern void printeventmode();
-extern void Wprintf(char *);
 extern void popupprompt(xcWidget, char *, char *, void (*function)(),
                         buttonsave *, const char *);
 extern void getproptext(xcWidget, propstruct *, caddr_t);
@@ -851,13 +856,12 @@ extern int xc_getlayoutcolor(int);
 extern int query_named_color(char *);
 extern void outputpopup(xcWidget, caddr_t, caddr_t);
 extern void docommand();
-extern void W1printf(char *);
-extern void W2printf(char *);
 extern void destroypopup(xcWidget, popupstruct *, caddr_t);
 extern int xc_alloccolor(char *);
 extern void dointr(int);
 extern void DoNothing(xcWidget, caddr_t, caddr_t);
 extern u_short countchanges(char **);
+extern u_short getchanges(objectptr);
 extern void quitcheck(xcWidget, caddr_t, caddr_t);
 extern void quit(xcWidget, caddr_t);
 extern void resizetoolbar();
