@@ -45,9 +45,8 @@
 
 extern char _STR2[250], _STR[150];
 extern Globaldata xobjs;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Display *dpy;
-extern short eventmode;
 extern int *appcolors;
 extern int number_colors;
 extern colorindex *colorlist;
@@ -90,13 +89,13 @@ void ghostinit_local()
 {
    sprintf(_STR, "%ld %d %d %d %d %d %g %g %d %d %d %d",
 		0L, 0, 0, 0,
-		areastruct.width * 75 / 72,
-		areastruct.height * 75 / 72,
+		areawin->width * 75 / 72,
+		areawin->height * 75 / 72,
 		75.0, 75.0, 0, 0, 0, 0);
-   XChangeProperty(dpy, areastruct.areawin, gv, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(dpy, areawin->window, gv, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    sprintf(_STR, "%s %d %d", "Color", (int)FOREGROUND, (int)BACKGROUND);
-   XChangeProperty(dpy, areastruct.areawin, gvc, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(dpy, areawin->window, gvc, XA_STRING, 8, PropModeReplace,
 	_STR, strlen(_STR));
    XSync(dpy, False);
 
@@ -120,13 +119,13 @@ Boolean render_client(XEvent *eventPtr)
 #endif
       mwin = eventPtr->xclient.data.l[0];
       Wprintf("Background finished.");
-      XDefineCursor(dpy, areastruct.areawin, DEFAULTCURSOR);
+      XDefineCursor(dpy, areawin->window, DEFAULTCURSOR);
 
       /* Mark this as the most recently rendered background, so we don't	*/
       /* have to render more than necessary.					*/
-      areastruct.lastbackground = xobjs.pagelist[areastruct.page]->background.name;
+      areawin->lastbackground = xobjs.pagelist[areawin->page]->background.name;
       gs_state = GS_READY;
-      drawarea(areastruct.area, NULL, NULL);
+      drawarea(areawin->area, NULL, NULL);
    }
    else if (eventPtr->xclient.message_type == gvdone) {
 #ifdef GS_DEBUG
@@ -187,7 +186,7 @@ void send_client(Atom msg)
 
    event.xclient.type = ClientMessage;
    event.xclient.display = dpy;
-   event.xclient.window = areastruct.areawin;
+   event.xclient.window = areawin->window;
    event.xclient.message_type = msg;
    event.xclient.format = 32;
    event.xclient.data.l[0] = mwin;
@@ -233,23 +232,23 @@ void start_gs()
 #ifdef TCL_WRAPPER
    if (bbuf != (Pixmap)NULL) Tk_FreePixmap(dpy, bbuf);
 #ifdef DOUBLEBUFFER
-   bbuf = Tk_GetPixmap(dpy, dbuf, areastruct.width, areastruct.height,
-		Tk_Depth(areastruct.area));
+   bbuf = Tk_GetPixmap(dpy, dbuf, areawin->width, areawin->height,
+		Tk_Depth(areawin->area));
 #else
-   bbuf = Tk_GetPixmap(dpy, (Drawable)areastruct.areawin,
-		areastruct.width, areastruct.height,
-		Tk_Depth(areastruct.area));
+   bbuf = Tk_GetPixmap(dpy, (Drawable)areawin->window,
+		areawin->width, areawin->height,
+		Tk_Depth(areawin->area));
 #endif
 
 #else	/* !TCL_WRAPPER */
    if (bbuf != (Pixmap)NULL) XFreePixmap(dpy, bbuf);
 #ifdef DOUBLEBUFFER
-   bbuf = XCreatePixmap(dpy, dbuf, areastruct.width, areastruct.height,
-		DefaultDepthOfScreen(xcScreen(areastruct.area))); 
+   bbuf = XCreatePixmap(dpy, dbuf, areawin->width, areawin->height,
+		DefaultDepthOfScreen(xcScreen(areawin->area))); 
 #else
-   bbuf = XCreatePixmap(dpy, areastruct.areawin,
-		areastruct.width, areastruct.height,
-		DefaultDepthOfScreen(xcScreen(areastruct.area))); 
+   bbuf = XCreatePixmap(dpy, areawin->window,
+		areawin->width, areawin->height,
+		DefaultDepthOfScreen(xcScreen(areawin->area))); 
 #endif	/* DOUBLEBUFFER */
 #endif  /* TCL_WRAPPER */
 
@@ -289,19 +288,19 @@ void start_gs()
    	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
 	 putenv(env_str1);
 #ifdef DOUBLEBUFFER
-   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areastruct.areawin, (long)bbuf);
+   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areawin->window, (long)bbuf);
 	 putenv(env_str2);
 #else
-	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areastruct.areawin);
+	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areawin->window);
 	 putenv(env_str2);
 #endif
 #else
 	 setenv("DISPLAY", XDisplayString(dpy), True);
 #ifdef DOUBLEBUFFER
-	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
+	 sprintf(_STR, "%ld %ld", (long)areawin->window, (long)bbuf);
 	 setenv("GHOSTVIEW", _STR, True);
 #else
- 	 sprintf(_STR, "%ld", (long)areastruct.areawin);
+ 	 sprintf(_STR, "%ld", (long)areawin->window);
 	 setenv("GHOSTVIEW", _STR, True);
 #endif
 #endif
@@ -329,7 +328,7 @@ void parse_bg(FILE *fi, FILE *fbg) {
    char line_in[256];
    float psscale;
 
-   psscale = getpsscale(xobjs.pagelist[areastruct.page]->outscale, areastruct.page);
+   psscale = getpsscale(xobjs.pagelist[areawin->page]->outscale, areawin->page);
 
    for(;;) {
       if (fgets(line_in, 255, fi) == NULL) {
@@ -357,10 +356,10 @@ void parse_bg(FILE *fi, FILE *fbg) {
 			llx, lly, urx, ury);
 #endif
 	
-	       xobjs.pagelist[areastruct.page]->background.bbox.lowerleft.x = llx;
-	       xobjs.pagelist[areastruct.page]->background.bbox.lowerleft.y = lly;
-	       xobjs.pagelist[areastruct.page]->background.bbox.width = (urx - llx);
-	       xobjs.pagelist[areastruct.page]->background.bbox.height = (ury - lly);
+	       xobjs.pagelist[areawin->page]->background.bbox.lowerleft.x = llx;
+	       xobjs.pagelist[areawin->page]->background.bbox.lowerleft.y = lly;
+	       xobjs.pagelist[areawin->page]->background.bbox.width = (urx - llx);
+	       xobjs.pagelist[areawin->page]->background.bbox.height = (ury - lly);
 	       if (fbg == (FILE *)NULL) break;
 	    }
 	 }
@@ -378,7 +377,7 @@ void bg_get_bbox()
    FILE *fi;
    char *fname;
 
-   fname = xobjs.pagelist[areastruct.page]->background.name;
+   fname = xobjs.pagelist[areawin->page]->background.name;
    if ((fi = fopen(fname, "r")) == NULL) {
       fprintf(stderr, "Failure to open background file to get bounding box info\n");
       return;
@@ -491,9 +490,9 @@ void register_bg(char *gsfile)
    else
       reset_gs();
 
-   xobjs.pagelist[areastruct.page]->background.name =
+   xobjs.pagelist[areawin->page]->background.name =
 		(char *) malloc(strlen(gsfile) + 1);
-   strcpy(xobjs.pagelist[areastruct.page]->background.name, gsfile);
+   strcpy(xobjs.pagelist[areawin->page]->background.name, gsfile);
 }
 
 /*------------------------------------------------------*/
@@ -509,7 +508,7 @@ void loadbackground()
    register_bg(_STR2);
    bg_get_bbox();
    updatepagebounds(topobject);
-   zoomview(areastruct.area, NULL, NULL);
+   zoomview(areawin->area, NULL, NULL);
 }
 
 /*------------------------------------------------------*/
@@ -537,29 +536,29 @@ int renderbackground()
 
    if (gsproc < 0) return -1;
 
-   defscale = (xobjs.pagelist[areastruct.page]->coordstyle == CM) ?
+   defscale = (xobjs.pagelist[areawin->page]->coordstyle == CM) ?
 	CMSCALE : INCHSCALE;
 
-   psnorm = (*areastruct.vscale) * (1.0 / defscale) * devres;
+   psnorm = (*areawin->vscale) * (1.0 / defscale) * devres;
 
-   psxpos = (float)(-areastruct.pcorner->x) * (*areastruct.vscale) * devres;
-   psypos = (float)(-areastruct.pcorner->y) * (*areastruct.vscale) * devres
-		+ ((float)areastruct.height / 12.0);
+   psxpos = (float)(-areawin->pcorner->x) * (*areawin->vscale) * devres;
+   psypos = (float)(-areawin->pcorner->y) * (*areawin->vscale) * devres
+		+ ((float)areawin->height / 12.0);
 
    /* Conditions for re-rendering:  Must have a background specified */
    /* and must be on the page, not a library or other object.	     */
 
-   if (xobjs.pagelist[areastruct.page]->background.name == (char *)NULL)
+   if (xobjs.pagelist[areawin->page]->background.name == (char *)NULL)
       return -1;
-   else if (areastruct.lastbackground
-		== xobjs.pagelist[areastruct.page]->background.name) {
+   else if (areawin->lastbackground
+		== xobjs.pagelist[areawin->page]->background.name) {
       return 0;
    }
 
    if (is_page(topobject) == -1)
       return -1;
 
-   bgfile = xobjs.pagelist[areastruct.page]->background.name;
+   bgfile = xobjs.pagelist[areawin->page]->background.name;
    if (*bgfile == '@') bgfile++;
 
    /* Ask ghostscript to produce the next page */
@@ -568,7 +567,7 @@ int renderbackground()
    /* Set the last background name to NULL; this will get the 	*/
    /* value when the rendering is done.				*/
 
-   areastruct.lastbackground = NULL;
+   areawin->lastbackground = NULL;
 
    /* write scale and position to ghostscript 		*/
    /* and tell ghostscript to run the requested file	*/
@@ -589,7 +588,7 @@ int renderbackground()
    fprintf(stdout, "Rendering background from file \"%s\"\n", bgfile);
 #endif
    Wprintf("Rendering background image.");
-   XDefineCursor(dpy, areastruct.areawin, WAITFOR);
+   XDefineCursor(dpy, areawin->window, WAITFOR);
 
    /* The page will be refreshed when we receive confirmation	*/
    /* from ghostscript that the buffer has been rendered.	*/
@@ -612,11 +611,11 @@ int copybackground()
       return -1;
 
 #ifdef DOUBLEBUFFER
-   XCopyArea(dpy, bbuf, dbuf, areastruct.gc, 0, 0,
-             areastruct.width, areastruct.height, 0, 0);
+   XCopyArea(dpy, bbuf, dbuf, areawin->gc, 0, 0,
+             areawin->width, areawin->height, 0, 0);
 #else
-   XCopyArea(dpy, bbuf, areastruct.areawin, areastruct.gc, 0, 0,
-             areastruct.width, areastruct.height, 0, 0);
+   XCopyArea(dpy, bbuf, areawin->window, areawin->gc, 0, 0,
+             areawin->width, areawin->height, 0, 0);
 #endif
 
    return 0;
