@@ -43,8 +43,7 @@ extern Tcl_Interp *xcinterp;
 #endif
 
 extern Globaldata xobjs;
-extern Clientdata areastruct;
-extern short	  eventmode;
+extern XCWindowData *areawin;
 extern int	  *appcolors;
 extern xcWidget     menuwidgets[];
 extern xcWidget	  netbutton;
@@ -91,8 +90,8 @@ void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
 {
    Arg wargs[1];
 
-   if (areastruct.schemon) {
-      areastruct.schemon = False;
+   if (areawin->schemon) {
+      areawin->schemon = False;
 #ifndef TCL_WRAPPER
       XtUnmanageChild(wschema);
       XtUnmanageChild(wsymb);
@@ -103,7 +102,7 @@ void doxschema(xcWidget w, pointertype mode, caddr_t calldata)
 #endif
    }
    else {
-      areastruct.schemon = True;
+      areawin->schemon = True;
 #ifndef TCL_WRAPPER
       XtManageChild(wschema);
       XtManageChild(wsymb);
@@ -229,7 +228,7 @@ int checkpagename(objectptr thispageobj)
 
 void startconnect(xcWidget button, caddr_t clientdata, caddr_t calldata)
 {
-   if (areastruct.selects > 0)
+   if (areawin->selects > 0)
       connectivity(button, clientdata, calldata);
 }
 
@@ -252,17 +251,17 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    stringpart *ppin;
 
    /* erase any existing highlighted network */
-   highlightnetlist(topobject, areastruct.topinstance, 0);
+   highlightnetlist(topobject, areawin->topinstance, 0);
 
    seltop = (pushlistptr)malloc(sizeof(pushlist));
-   seltop->thisinst = areastruct.topinstance;
+   seltop->thisinst = areawin->topinstance;
    seltop->next = NULL;
 
    /* pick the first selection that looks like a valid network part */
 
-   if (areastruct.selects > 0) {
-      for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
-		areastruct.selects; gsel++) {
+   if (areawin->selects > 0) {
+      for (gsel = areawin->selectlist; gsel < areawin->selectlist +
+		areawin->selects; gsel++) {
 	 ggen = *(topobject->plist + *gsel);
          if (SELECTTYPE(gsel) == LABEL) {
 	    labelptr glab = SELTOLABEL(gsel);
@@ -274,8 +273,8 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	 }
       }
    }
-   if ((areastruct.selects == 0) || (gsel == areastruct.selectlist
-		+ areastruct.selects)) {
+   if ((areawin->selects == 0) || (gsel == areawin->selectlist
+		+ areawin->selects)) {
       rselect = recurselect(POLYGON | LABEL | OBJINST, MODE_CONNECT, &seltop);
       /* Should look at the top scorer, when scoring has been implemented */
       if (rselect && (rselect->selects > 0)) {
@@ -299,7 +298,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
    if (ggen != NULL) {
       if (checkvalid(topobject) == -1) {
          destroynets(topobject);
-         createnets(areastruct.topinstance);
+         createnets(areawin->topinstance);
       }
       if ((netlist = is_resolved(&ggen, seltop, &nettop)) != NULL) {
          depth = pushnetwork(seltop, nettop);
@@ -323,7 +322,7 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 
 	 if (netlist->subnets == 0) {
 	    ppin = nettopin(netlist->net.id, pschem, NULL);
-            snew = textprint(ppin, areastruct.topinstance);
+            snew = textprint(ppin, areawin->topinstance);
 	    sprintf(_STR2, "Network is \"%s\" in %s", snew, nettop->name);
             free(snew);
 	 }
@@ -336,14 +335,14 @@ void connectivity(xcWidget button, caddr_t clientdata, caddr_t calldata)
 	       netid = sbus->netid;
 	       subnetid = sbus->subnetid;
 	       ppin = nettopin(netid, pschem, NULL);
-               snew = textprintsubnet(ppin, areastruct.topinstance, subnetid);
+               snew = textprintsubnet(ppin, areawin->topinstance, subnetid);
 	       sprintf(sptr, "%s ", snew);
 	       sptr += strlen(snew) + 1;
 	       free(snew);
 	    }
 	    sprintf(sptr, "in %s", nettop->name);
 	 }
-	 Wprintf(_STR2);
+	 Wprintf("%s", _STR2);
 
 #ifdef TCL_WRAPPER
 	 Tcl_SetObjResult(xcinterp, Tcl_NewStringObj(snew, strlen(snew)));
@@ -438,7 +437,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
    char typestr[40];
    short savetype = -1;
 
-   if (areastruct.selects == 0) {
+   if (areawin->selects == 0) {
       Wprintf("Must first select a label to change type");
       return;
    }
@@ -459,8 +458,8 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
          break;
    }
 
-   for (gsel = areastruct.selectlist; gsel < areastruct.selectlist +
-		areastruct.selects; gsel++)
+   for (gsel = areawin->selectlist; gsel < areawin->selectlist +
+		areawin->selects; gsel++)
       if (SELECTTYPE(gsel) == LABEL) {
 	 labelptr glab = SELTOLABEL(gsel);
 	 savetype = glab->pin;
@@ -471,7 +470,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
    if (savetype >= 0) {
       unselect_all();
       drawarea(NULL, NULL, NULL);
-      Wprintf(typestr);
+      Wprintf("%s", typestr);
    }
    else {
       Wprintf("No labels selected.");
@@ -678,49 +677,6 @@ void collectsubschems(int toppage)
    free((char *)pagelist);
 }
 
-/*--------------------------------------------------------*/
-/* Copy label to corresponding schematic/symbol		  */
-/*--------------------------------------------------------*/
-
-void copypinlabel(labelptr pinlab)
-{
-   labelptr *newlabel, tlabel;
-   genericptr *tgen;
-   objectptr schemobj = topobject->symschem;
-
-   if (areastruct.schemon && schemobj != NULL && (pinlab->pin == LOCAL)) {
-
-      /* If this pin already exists, don't copy it */
-
-      for (tgen = schemobj->plist; tgen < schemobj->plist + schemobj->parts;
-		tgen++) {
-         if (IS_LABEL(*tgen)) {
-	    tlabel = TOLABEL(tgen);
-	    if (!stringcomp(tlabel->string, pinlab->string)) return;
-         }
-      }
-
-      NEW_LABEL(newlabel, schemobj);
-      (*newlabel)->pin = pinlab->pin;
-      (*newlabel)->rotation = pinlab->rotation;
-      (*newlabel)->justify = pinlab->justify;
-      (*newlabel)->color = pinlab->color;
-      (*newlabel)->scale = pinlab->scale;
-      (*newlabel)->string = stringcopy(pinlab->string);
-      (*newlabel)->passed = NULL;
-
-      /* place label just outside bounding box, then recompute bbox */
-
-      (*newlabel)->position.x = schemobj->bbox.lowerleft.x + (schemobj->bbox.width >> 1);
-      (*newlabel)->position.y = schemobj->bbox.lowerleft.y - TEXTHEIGHT
-	   * (*newlabel)->scale;
-      schemobj->parts++;
-      incr_changes(schemobj);
-      invalidate_netlist(schemobj);
-      singlebbox(ENDPART);
-   }
-}
-
 /*-------------------------------------------------------*/
 /* Check if top-level page is the same name as a library */
 /* object; if so, connect it.				 */
@@ -736,7 +692,7 @@ int checkschem(objectptr thisobj, char *cname)
    objectptr *tlib;
    short i, j;
 
-   if ((areastruct.schemon == False) || (thisobj->symschem != NULL)) return 0;
+   if ((areawin->schemon == False) || (thisobj->symschem != NULL)) return 0;
 
    for (i = 0; i < xobjs.numlibs; i++) {
       for (j = 0; j < xobjs.userlibs[i].number; j++) {
@@ -764,7 +720,7 @@ int checksym(objectptr symobj, char *cname)
    short cpage;
    objectptr checkpage;
 
-   if (areastruct.schemon == False || symobj->symschem != NULL) return 0;
+   if (areawin->schemon == False || symobj->symschem != NULL) return 0;
 
    for (cpage = 0; cpage < xobjs.pages; cpage++) {
       if (xobjs.pagelist[cpage]->pageinst != NULL) {
@@ -833,7 +789,7 @@ void swapschem(int allow_create, int libnum)
    pushlistptr stacktop;
    short loclibnum = libnum;
 
-   if (areastruct.schemon == False) return;
+   if (areawin->schemon == False) return;
    if (libnum == -1) loclibnum = USERLIB - LIBRARY;
 
    if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
@@ -854,11 +810,11 @@ void swapschem(int allow_create, int libnum)
 	 /* Push the current instance onto the push stack */
 	 /* Change the page without destroying the pushlist */
 
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 stacktop = areastruct.stack;
-	 areastruct.stack = NULL;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 stacktop = areawin->stack;
+	 areawin->stack = NULL;
          changepage(tpage);
-	 areastruct.stack = stacktop;
+	 areawin->stack = stacktop;
       }
       else {
 	 objectptr *newobject;
@@ -880,11 +836,11 @@ void swapschem(int allow_create, int libnum)
 	 /* Generate a library instance for this object and set the */
 	 /* top instance to point to it.			    */
 
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 areastruct.topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 areawin->topinstance = addtoinstlist(loclibnum, *newobject, FALSE);
 
 	 /* Generate the default bounding box for a size-zero object */
-	 calcbbox(areastruct.topinstance);
+	 calcbbox(areawin->topinstance);
       }
 
       /* set links between the two objects */
@@ -923,7 +879,7 @@ void swapschem(int allow_create, int libnum)
 	       (*pinlab)->color = lpin->color;
 	       (*pinlab)->rotation = 0;
 	       (*pinlab)->scale = 1.0;
-	       (*pinlab)->justify = areastruct.justify; 
+	       (*pinlab)->justify = areawin->justify; 
 	       (*pinlab)->position.x = 0;
 	       (*pinlab)->position.y = topobject->parts * (TEXTHEIGHT + 10);
 	       (*pinlab)->passed = NULL;
@@ -934,7 +890,7 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
       }
-      calcbbox(areastruct.topinstance);
+      calcbbox(areawin->topinstance);
 
       /* Recreate the user library with the new symbol */
       if (savepage->schemtype != SYMBOL) composelib(loclibnum + LIBRARY);
@@ -944,10 +900,10 @@ void swapschem(int allow_create, int libnum)
       /* If symschem matches the last entry on the push stack, then we	*/
       /* pop; otherwise, we push.					*/
 
-      if (areastruct.stack && areastruct.stack->thisinst->thisobject
+      if (areawin->stack && areawin->stack->thisinst->thisobject
 			== topobject->symschem) {
-	 areastruct.topinstance = areastruct.stack->thisinst;
-	 pop_stack(&areastruct.stack);
+	 areawin->topinstance = areawin->stack->thisinst;
+	 pop_stack(&areawin->stack);
       }
       else {
 	 int p;
@@ -983,8 +939,8 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
 	       
-	 push_stack(&areastruct.stack, areastruct.topinstance);
-	 areastruct.topinstance = syminst;
+	 push_stack(&areawin->stack, areawin->topinstance);
+	 areawin->topinstance = syminst;
       }
    }
 
@@ -1024,7 +980,7 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    /* Check if this function is valid */
 
-   if (areastruct.schemon == False) return;
+   if (areawin->schemon == False) return;
 
    /* Check for requirement to change the name before creating the symbol */
 
