@@ -7,10 +7,8 @@
 #include <stdlib.h>
 #include <ctype.h>   /* for isprint() and isdigit() */
 
-#ifndef _MSC_VER
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
-#endif
 
 /*------------------------------------------------------------------------*/
 /* Local includes                                                         */
@@ -33,12 +31,11 @@
 /*------------------------------------------------------------------------*/
 
 extern Display *dpy;
-extern Clientdata areastruct;
+extern XCWindowData *areawin;
 extern Globaldata xobjs;
 extern short textpos, textend;
 extern short fontcount;
 extern fontinfo *fonts;
-extern short eventmode;
 extern int *appcolors;
 extern colorindex *colorlist;
 extern char _STR[150];
@@ -92,39 +89,39 @@ void joinlabels()
    int tpos;
    labelptr dest, source;
 
-   if (areastruct.selects < 2) {
+   if (areawin->selects < 2) {
       Wprintf("Not enough labels selected for joining");
       return;
    }
 
    XcSetFunction(GXcopy);
-   XSetForeground(dpy, areastruct.gc, BACKGROUND);
+   XSetForeground(dpy, areawin->gc, BACKGROUND);
 
-   for (jl = areastruct.selectlist; jl < areastruct.selectlist +
-		areastruct.selects; jl++) {
+   for (jl = areawin->selectlist; jl < areawin->selectlist +
+		areawin->selects; jl++) {
       if (SELECTTYPE(jl) == LABEL) {
 	 dest = SELTOLABEL(jl);
-	 UDrawString(dest, DOFORALL, areastruct.topinstance);
+	 UDrawString(dest, DOFORALL, areawin->topinstance);
 	 for (endpart = dest->string; endpart->nextpart != NULL; endpart =
 			endpart->nextpart);
 	 break;
       }
    }
       
-   for (++jl; jl < areastruct.selectlist + areastruct.selects; jl++) {
+   for (++jl; jl < areawin->selectlist + areawin->selects; jl++) {
       if (SELECTTYPE(jl) == LABEL) {
 	 source = SELTOLABEL(jl);
-	 UDrawString(source, DOFORALL, areastruct.topinstance);
+	 UDrawString(source, DOFORALL, areawin->topinstance);
 	 endpart->nextpart = source->string;
 	 for (; endpart->nextpart != NULL; endpart = endpart->nextpart);
 	 free(source);
 	 removep(jl, 0);
-         reviseselect(areastruct.selectlist, areastruct.selects, jl);
+         reviseselect(areawin->selectlist, areawin->selects, jl);
       }
    }
 
-   XSetForeground(dpy, areastruct.gc, dest->color);
-   UDrawString(dest, dest->color, areastruct.topinstance);
+   XSetForeground(dpy, areawin->gc, dest->color);
+   UDrawString(dest, dest->color, areawin->topinstance);
 
    incr_changes(topobject);
    clearselects();
@@ -147,12 +144,12 @@ stringpart *makesegment(stringpart **strhead, stringpart *before)
    }
    else {				/* otherwise */
       for(lastptr = *strhead; lastptr != NULL;) {
-	 nextptr = nextstringpart(lastptr, areastruct.topinstance);
+	 nextptr = nextstringpart(lastptr, areawin->topinstance);
 	 if (nextptr == before) {
 	    if (lastptr->type == PARAM_START) {
 	       oparamptr obs = NULL;
 	       char *key = lastptr->data.string;
-	       obs = find_param(areastruct.topinstance, key);
+	       obs = find_param(areawin->topinstance, key);
 	       if (obs == NULL) {
 		  Wprintf("Error:  Bad parameter!");
 		  return NULL;
@@ -627,7 +624,7 @@ char *textprintsubnet(stringpart *strtop, objinstptr localinst, int subnet)
 
    newstr = xcstringtostring(strtop, localinst, True);
    if (subnet >= 0) {
-      busptr = strchr(newstr, areastruct.buschar);
+      busptr = strchr(newstr, areawin->buschar);
       if (busptr != NULL) {
 	 endptr = find_delimiter(busptr);
 	 if (endptr != NULL) {
@@ -648,8 +645,8 @@ char *textprintsubnet(stringpart *strtop, objinstptr localinst, int subnet)
 	 strcpy(substr, newstr);
 	 endptr = substr;
 	 while ((*endptr) != '\0') endptr++;
-	 sprintf(endptr, "%c%d%c", areastruct.buschar, subnet,
-		standard_delimiter_end(areastruct.buschar));
+	 sprintf(endptr, "%c%d%c", areawin->buschar, subnet,
+		standard_delimiter_end(areawin->buschar));
 	 free(newstr);
 	 return substr;
       }
@@ -679,7 +676,7 @@ char *textprintnet(char *prefix, char *pinstring, Genericlist *sublist)
    else {	/* just make a comma-separated list */
       newstr = (char *)malloc(strlen(prefix) + 20 + 3 * sublist->subnets);
       sbus = sublist->net.list;
-      sprintf(newstr, "%s%d%c", prefix, sbus->netid, areastruct.buschar);
+      sprintf(newstr, "%s%d%c", prefix, sbus->netid, areawin->buschar);
       for (i = 0; i < sublist->subnets; i++) {
 	 sbus = sublist->net.list + i;
 	 sptr = newstr + strlen(newstr);
@@ -688,7 +685,7 @@ char *textprintnet(char *prefix, char *pinstring, Genericlist *sublist)
 	 sprintf(sptr, "%d", sbus->subnetid);
       }
       sptr = newstr + strlen(newstr);
-      sprintf(sptr, "%c", standard_delimiter_end(areastruct.buschar));
+      sprintf(sptr, "%c", standard_delimiter_end(areawin->buschar));
    }
    return newstr;
 }
@@ -823,7 +820,7 @@ int sub_bus_idx(labelptr thislab, objinstptr thisinst)
    for (strptr = thislab->string; strptr != NULL; strptr =
 		nextstringpart(strptr, thisinst)) {
       if (strptr->type == TEXT_STRING) {
-	 if ((busptr = strchr(strptr->data.string, areastruct.buschar)) != NULL) {
+	 if ((busptr = strchr(strptr->data.string, areawin->buschar)) != NULL) {
 	    if (sscanf(++busptr, "%d", &busidx) == 1)
 	       return busidx;
 	 }
@@ -853,7 +850,7 @@ Boolean pin_is_bus(labelptr thislab, objinstptr thisinst)
    for (strptr = thislab->string; strptr != NULL; strptr =
 		nextstringpart(strptr, thisinst)) {
       if (strptr->type == TEXT_STRING) {
-	 if ((busptr = strchr(strptr->data.string, areastruct.buschar)) != NULL) {
+	 if ((busptr = strchr(strptr->data.string, areawin->buschar)) != NULL) {
 	    if (isdigit(*(++busptr)))
 	       return TRUE;
 	    else
@@ -908,7 +905,7 @@ Genericlist *break_up_bus(labelptr blab, objinstptr thisinst, Genericlist *netli
 
    tptr = textprint(blab->string, thisinst);
    tlen = strlen(tptr) + 1;
-   buspos = strchr(tptr, areastruct.buschar);
+   buspos = strchr(tptr, areawin->buschar);
 
    /* The notation "(...)" with NO TEXT preceding the opening bus	*/
    /* delimiter, is assumed to represent a numerical pin range.  So	*/
@@ -1037,8 +1034,8 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2,
          switch (strptr1->type) {
 	    case TEXT_STRING:
 	       if (in_bus == 1) {
-		  char matchchar = areastruct.buschar;
-		  switch (areastruct.buschar) {
+		  char matchchar = areawin->buschar;
+		  switch (areawin->buschar) {
 		     case '(': matchchar = ')'; break;
 		     case '[': matchchar = ']'; break;
 		     case '{': matchchar = '}'; break;
@@ -1067,7 +1064,7 @@ int stringcomprelaxed(stringpart *string1, stringpart *string2,
 	       /* except between the bus notation delimiters (default	*/
 	       /* "()".							*/
 		
-	       buspos = strchr(strptr1->data.string, areastruct.buschar);
+	       buspos = strchr(strptr1->data.string, areawin->buschar);
 	       if (buspos != NULL) {
 
 		  bpos = (int)(buspos - (char *)(strptr1->data.string)) + 1;
@@ -1353,7 +1350,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    int    defaultcolor, curcolor, scolor;
    short  oldx, oldfont, oldstyle;
    float  tmpscale = 1.0, natscale = 1.0;
-   float  tmpthick = xobjs.pagelist[areastruct.page]->wirewidth;
+   float  tmpthick = xobjs.pagelist[areawin->page]->wirewidth;
    XPoint newpoint, bboxin[2], bboxout[2];
    u_char xm, ym;
    TextExtents tmpext;
@@ -1402,7 +1399,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    /* them from the circuit point to which they attach.  */
 
    if (drawlabel->pin) {
-      if (!areastruct.schemon)
+      if (!areawin->schemon)
 	 goto enddraw;	/* Don't draw pins when schematic capture is off */
       else
 	 pinadjust(tmpjust, &(newpoint.x), &(newpoint.y), 1);
@@ -1421,7 +1418,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
    xm = (bboxout[0].x < bboxout[1].x) ? 0 : 1;
    ym = (bboxout[0].y < bboxout[1].y) ? 0 : 1;
 
-   if (bboxout[xm].x < areastruct.width && bboxout[ym].y < areastruct.height &&
+   if (bboxout[xm].x < areawin->width && bboxout[ym].y < areawin->height &&
        bboxout[1 - xm].x > 0 && bboxout[1 - ym].y > 0) {
 
        pos = 0;
@@ -1447,7 +1444,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 		
 		/* simple boldface technique for derived fonts */
 
-		xobjs.pagelist[areastruct.page]->wirewidth =
+		xobjs.pagelist[areawin->page]->wirewidth =
    		   ((fonts[ffont].flags & 0x21) == 0x21) ?  4.0 : 2.0;
 		break;
 
@@ -1597,13 +1594,13 @@ enddraw:
 
    UPopCTM();
 
-   if (drawlabel->pin && areastruct.schemon) UDrawXDown(drawlabel);
+   if (drawlabel->pin && areawin->schemon) UDrawXDown(drawlabel);
 
    if ((defaultcolor != DOFORALL) && (passcolor != curcolor)) {
       XTopSetForeground(passcolor);
    }
 
-   xobjs.pagelist[areastruct.page]->wirewidth = tmpthick;
+   xobjs.pagelist[areawin->page]->wirewidth = tmpthick;
 }
 
 /*----------------------------------------------------------------------*/
@@ -1756,16 +1753,16 @@ TextExtents ULength(stringpart *string, objinstptr localinst, float newscale,
 
 void undrawtextsimple(labelptr settext)
 {
-   SetFunction(dpy, areastruct.gc, GXcopy);
+   SetFunction(dpy, areawin->gc, GXcopy);
    XTopSetForeground(BACKGROUND);
-   UDrawString(settext, DOFORALL, areastruct.topinstance);
+   UDrawString(settext, DOFORALL, areawin->topinstance);
 }
 
 /*----------------------------------------------------------------------*/
 
 void redrawtextsimple(labelptr settext)
 {
-   UDrawString(settext, settext->color, areastruct.topinstance);
+   UDrawString(settext, settext->color, areawin->topinstance);
 }
 
 /*----------------------------------------------------------------------*/
@@ -1856,7 +1853,7 @@ void composefontlib(short cfont)
    directory->parts = 0;
 
    /* 0.5 is the default vscale;  16 is no. characters per line */
-   del = min(areastruct.width, areastruct.height) / (0.5 * 16);
+   del = min(areawin->width, areawin->height) / (0.5 * 16);
    qdel = del >> 2;
 
    for (i = 0; i < 256; i++) {
@@ -1929,10 +1926,10 @@ void fontcat_op(int op, int x, int y)
 
    if (op != XCF_Cancel) {
 
-      window_to_user(x, y, &areastruct.save);
+      window_to_user(x, y, &areawin->save);
  
-      chy = -areastruct.save.y / del + 1;
-      chx = areastruct.save.x / del;
+      chy = -areawin->save.y / del + 1;
+      chx = areawin->save.x / del;
 
       chx = min(15, chx);
       chy = min(15, chy);
