@@ -1168,7 +1168,7 @@ enum {
    XCF_ENDDATA = -2,		XCF_SPACER  /* -1 */,
    XCF_Page	 /* 0 */,	XCF_Justify /* 1 */,
    XCF_Superscript /* 2 */,	XCF_Subscript /* 3 */,
-   XCF_Normalscript /* 4 */,	XCF_Nextfont /* 5 */,
+   XCF_Normalscript /* 4 */,	XCF_Font /* 5 */,
    XCF_Boldfont /* 6 */,	XCF_Italicfont /* 7 */,
    XCF_Normalfont /* 8 */,	XCF_Underline /* 9 */,
    XCF_Overline /* 10 */,	XCF_ISO_Encoding /* 11 */,
@@ -1220,7 +1220,8 @@ enum {
    XCF_Finish /* 102 */,	XCF_Cancel_Last /* 103 */,
    XCF_Sim /* 104 */,		XCF_SPICE /* 105 */,
    XCF_PCB /* 106 */,		XCF_SPICEflat /* 107 */,
-   XCF_Text_Delete_Param /* 108 */,
+   XCF_Color /* 108 */,		XCF_Text_Delete_Param /* 109 */,
+   XCF_Scale /* 110 */,		XCF_ChangeStyle /* 111 */,
    NUM_FUNCTIONS
 };
 
